package com.link.core.mybatis;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Properties;
import javax.sql.DataSource;
import org.apache.ibatis.builder.xml.XMLConfigBuilder;
import org.apache.ibatis.builder.xml.XMLMapperBuilder;
import org.apache.ibatis.executor.ErrorContext;
import org.apache.ibatis.mapping.DatabaseIdProvider;
import org.apache.ibatis.mapping.Environment;
import org.apache.ibatis.plugin.Interceptor;
import org.apache.ibatis.reflection.factory.ObjectFactory;
import org.apache.ibatis.reflection.wrapper.ObjectWrapperFactory;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import org.apache.ibatis.transaction.TransactionFactory;
import org.apache.ibatis.type.TypeHandler;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.transaction.SpringManagedTransactionFactory;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.core.NestedIOException;
import org.springframework.core.io.Resource;
import org.springframework.jdbc.datasource.TransactionAwareDataSourceProxy;
import org.springframework.util.Assert;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

public class HotSqlSessionFactoryBean implements FactoryBean<SqlSessionFactory>, InitializingBean, ApplicationListener<ApplicationEvent> {
   private Resource configLocation;
   private Resource[] mapperLocations;
   private DataSource dataSource;
   private TransactionFactory transactionFactory;
   private Properties configurationProperties;
   private SqlSessionFactoryBuilder sqlSessionFactoryBuilder = new SqlSessionFactoryBuilder();
   private SqlSessionFactory sqlSessionFactory;
   private String environment = SqlSessionFactoryBean.class.getSimpleName();
   private boolean failFast;
   private Interceptor[] plugins;
   private TypeHandler<?>[] typeHandlers;
   private String typeHandlersPackage;
   private Class<?>[] typeAliases;
   private String typeAliasesPackage;
   private Class<?> typeAliasesSuperType;
   private DatabaseIdProvider databaseIdProvider;
   private ObjectFactory objectFactory;
   private ObjectWrapperFactory objectWrapperFactory;

   public void setObjectFactory(ObjectFactory objectFactory) {
      this.objectFactory = objectFactory;
   }

   public void setObjectWrapperFactory(ObjectWrapperFactory objectWrapperFactory) {
      this.objectWrapperFactory = objectWrapperFactory;
   }

   public DatabaseIdProvider getDatabaseIdProvider() {
      return this.databaseIdProvider;
   }

   public void setDatabaseIdProvider(DatabaseIdProvider databaseIdProvider) {
      this.databaseIdProvider = databaseIdProvider;
   }

   public void setPlugins(Interceptor[] plugins) {
      this.plugins = plugins;
   }

   public void setTypeAliasesPackage(String typeAliasesPackage) {
      this.typeAliasesPackage = typeAliasesPackage;
   }

   public void setTypeAliasesSuperType(Class<?> typeAliasesSuperType) {
      this.typeAliasesSuperType = typeAliasesSuperType;
   }

   public void setTypeHandlersPackage(String typeHandlersPackage) {
      this.typeHandlersPackage = typeHandlersPackage;
   }

   public void setTypeHandlers(TypeHandler<?>[] typeHandlers) {
      this.typeHandlers = typeHandlers;
   }

   public void setTypeAliases(Class<?>[] typeAliases) {
      this.typeAliases = typeAliases;
   }

   public void setFailFast(boolean failFast) {
      this.failFast = failFast;
   }

   public void setConfigLocation(Resource configLocation) {
      this.configLocation = configLocation;
   }

   public void setMapperLocations(Resource[] mapperLocations) {
      this.mapperLocations = mapperLocations;
   }

   public void setConfigurationProperties(Properties sqlSessionFactoryProperties) {
      this.configurationProperties = sqlSessionFactoryProperties;
   }

   public void setDataSource(DataSource dataSource) {
      if (dataSource instanceof TransactionAwareDataSourceProxy) {
         this.dataSource = ((TransactionAwareDataSourceProxy)dataSource).getTargetDataSource();
      } else {
         this.dataSource = dataSource;
      }

   }

   public void setSqlSessionFactoryBuilder(SqlSessionFactoryBuilder sqlSessionFactoryBuilder) {
      this.sqlSessionFactoryBuilder = sqlSessionFactoryBuilder;
   }

   public void setTransactionFactory(TransactionFactory transactionFactory) {
      this.transactionFactory = transactionFactory;
   }

   public void setEnvironment(String environment) {
      this.environment = environment;
   }

   public void afterPropertiesSet() throws Exception {
      Assert.notNull(this.dataSource, "Property 'dataSource' is required");
      Assert.notNull(this.sqlSessionFactoryBuilder, "Property 'sqlSessionFactoryBuilder' is required");
      this.sqlSessionFactory = this.buildSqlSessionFactory();
   }

   protected SqlSessionFactory buildSqlSessionFactory() throws IOException {
      XMLConfigBuilder xmlConfigBuilder = null;
      Configuration configuration;
      if (this.configLocation != null) {
         xmlConfigBuilder = new XMLConfigBuilder(this.configLocation.getInputStream(), (String)null, this.configurationProperties);
         configuration = xmlConfigBuilder.getConfiguration();
      } else {
         configuration = new Configuration();
         configuration.setVariables(this.configurationProperties);
      }

      if (this.objectFactory != null) {
         configuration.setObjectFactory(this.objectFactory);
      }

      if (this.objectWrapperFactory != null) {
         configuration.setObjectWrapperFactory(this.objectWrapperFactory);
      }

      String[] typeHandlersPackageArray;
      String[] var4;
      int var5;
      int var6;
      String packageToScan;
      if (StringUtils.hasLength(this.typeAliasesPackage)) {
         typeHandlersPackageArray = StringUtils.tokenizeToStringArray(this.typeAliasesPackage, ",; \t\n");
         var4 = typeHandlersPackageArray;
         var5 = typeHandlersPackageArray.length;

         for(var6 = 0; var6 < var5; ++var6) {
            packageToScan = var4[var6];

            try {
               configuration.getTypeAliasRegistry().registerAliases(packageToScan, this.typeAliasesSuperType == null ? Object.class : this.typeAliasesSuperType);
            } catch (Exception var38) {
               Exception ex = var38;
               throw new NestedIOException("Scanned package: '" + packageToScan + "' for aliases", ex);
            } finally {
               ErrorContext.instance().reset();
            }
         }
      }

      int var42;
      if (!ObjectUtils.isEmpty(this.typeAliases)) {
         Class[] var40 = this.typeAliases;
         var42 = var40.length;

         for(var5 = 0; var5 < var42; ++var5) {
            Class<?> typeAlias = var40[var5];
            configuration.getTypeAliasRegistry().registerAlias(typeAlias);
         }
      }

      if (!ObjectUtils.isEmpty(this.plugins)) {
         Interceptor[] var41 = this.plugins;
         var42 = var41.length;

         for(var5 = 0; var5 < var42; ++var5) {
            Interceptor plugin = var41[var5];
            configuration.addInterceptor(plugin);
         }
      }

      if (StringUtils.hasLength(this.typeHandlersPackage)) {
         typeHandlersPackageArray = StringUtils.tokenizeToStringArray(this.typeHandlersPackage, ",; \t\n");
         var4 = typeHandlersPackageArray;
         var5 = typeHandlersPackageArray.length;

         for(var6 = 0; var6 < var5; ++var6) {
            packageToScan = var4[var6];
            configuration.getTypeHandlerRegistry().register(packageToScan);
         }
      }

      if (!ObjectUtils.isEmpty(this.typeHandlers)) {
         TypeHandler[] var43 = this.typeHandlers;
         var42 = var43.length;

         for(var5 = 0; var5 < var42; ++var5) {
            TypeHandler<?> typeHandler = var43[var5];
            configuration.getTypeHandlerRegistry().register(typeHandler);
         }
      }

      if (xmlConfigBuilder != null) {
         try {
            xmlConfigBuilder.parse();
         } catch (Exception var36) {
            throw new NestedIOException("Failed to parse config resource: " + this.configLocation, var36);
         } finally {
            ErrorContext.instance().reset();
         }
      }

      if (this.transactionFactory == null) {
         this.transactionFactory = new SpringManagedTransactionFactory();
      }

      configuration.setEnvironment(new Environment(this.environment, this.transactionFactory, this.dataSource));
      if (this.databaseIdProvider != null) {
         try {
            configuration.setDatabaseId(this.databaseIdProvider.getDatabaseId(this.dataSource));
         } catch (SQLException var35) {
            throw new NestedIOException("Failed getting a databaseId", var35);
         }
      }

      if (!ObjectUtils.isEmpty(this.mapperLocations)) {
         Resource[] var46 = this.mapperLocations;
         var42 = var46.length;

         for(var5 = 0; var5 < var42; ++var5) {
            Resource mapperLocation = var46[var5];
            if (mapperLocation != null) {
               try {
                  XMLMapperBuilder xmlMapperBuilder = new XMLMapperBuilder(mapperLocation.getInputStream(), configuration, mapperLocation.toString(), configuration.getSqlFragments());
                  xmlMapperBuilder.parse();
               } catch (Exception var33) {
                  throw new NestedIOException("Failed to parse mapping resource: '" + mapperLocation + "'", var33);
               } finally {
                  ErrorContext.instance().reset();
               }
            }
         }

         (new MapperRefresh(this.mapperLocations, configuration)).run();
      }

      return this.sqlSessionFactoryBuilder.build(configuration);
   }

   public SqlSessionFactory getObject() throws Exception {
      if (this.sqlSessionFactory == null) {
         this.afterPropertiesSet();
      }

      return this.sqlSessionFactory;
   }

   public Class<? extends SqlSessionFactory> getObjectType() {
      return this.sqlSessionFactory == null ? SqlSessionFactory.class : this.sqlSessionFactory.getClass();
   }

   public boolean isSingleton() {
      return true;
   }

   public void onApplicationEvent(ApplicationEvent event) {
      if (this.failFast && event instanceof ContextRefreshedEvent) {
         this.sqlSessionFactory.getConfiguration().getMappedStatementNames();
      }

   }
}
